<?php

namespace Drupal\config_auto_export;

use Drupal\Component\Datetime\Time;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Http\ClientFactory;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\Site\Settings;
use Drupal\Core\State\StateInterface;
use GuzzleHttp\Exception\GuzzleException;
use Drupal\Core\Session\AccountInterface;

class Service {

  public const STATE_KEY_DUE_TIMESTAMP = 'config_auto_export.due_next.timestamp';

  public const STATE_KEY_PAUSED = 'config_auto_export.paused';

  /** @var \Drupal\Core\Config\ImmutableConfig */
  protected $config;

  /**
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * @var \Drupal\Component\Datetime\Time
   */
  protected $time;

  /**
   * @var \Drupal\Core\Http\ClientFactory
   */
  protected $client;

  /**
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Service constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_channel_factory
   * @param \Drupal\Core\State\StateInterface $state
   * @param \Drupal\Component\Datetime\Time $time
   * @param \Drupal\Core\Http\ClientFactory $client
   * @param \Drupal\Core\Session\AccountInterface $account
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct(ConfigFactoryInterface $config_factory, FileSystemInterface $file_system,
                              LoggerChannelFactoryInterface $logger_channel_factory, StateInterface $state,
                              Time $time, ClientFactory $client, AccountInterface $account,
                              ModuleHandlerInterface $module_handler, EntityTypeManagerInterface $entity_type_manager) {
    $this->config = $config_factory->get('config_auto_export.settings');
    $this->fileSystem = $file_system;
    $this->logger = $logger_channel_factory->get('config');
    $this->state = $state;
    $this->time = $time;
    $this->client = $client;
    $this->account = $account;
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Pause config auto export.
   */
  public function pause(): void {
    $this->state->set(self::STATE_KEY_PAUSED, TRUE);
  }

  /**
   * Resume config auto export.
   */
  public function resume(): void {
    $this->state->set(self::STATE_KEY_PAUSED, FALSE);
  }

  /**
   * Is the process currently paused?
   *
   * @return bool
   */
  public function isPaused(): bool {
    return $this->state->get(self::STATE_KEY_PAUSED, FALSE);
  }

  /**
   * Trigger the webhook.
   *
   * @param bool $force
   *   Force trigger even if service is paused.
   *
   * @return bool
   */
  public function triggerExport(bool $force = FALSE): bool {
    if (!$force && $this->isPaused()) {
      return FALSE;
    }
    $webhook = $this->config->get('webhook');
    if (empty($webhook)) {
      return FALSE;
    }
    $exportPath = $this->fileSystem->realpath($this->config->get('directory'));
    $configPath = $this->fileSystem->realpath(Settings::get('config_sync_directory'));
    $currentUser = $this->account->getDisplayName();
    $configSplitPaths = [];
    if ($this->moduleHandler->moduleExists('config_split')) {
      /** @var \Drupal\config_split\Entity\ConfigSplitEntityInterface $split */
      foreach ($this->entityTypeManager->getStorage('config_split')->loadMultiple() as $split) {
        if ($split->status()) {
          $configSplitPaths[$split->id()] = $split->get('folder');
        }
      }
    }
    $data = [
      'form_params' => Yaml::decode(str_replace(
        ['[current user]', '[export directory]', '[config directory]', '[config split directories]'],
        [$currentUser, $exportPath, $configPath, json_encode($configSplitPaths)],
        $this->config->get('webhook_params'))),
    ];

    try {
      $client = $this->client->fromOptions(['base_uri' => $webhook]);
      $client->request('post', '', $data);
    }
    catch (GuzzleException $e) {
      $this->logger->critical('Trigger for config auto export failed: {msg}', ['msg' => $e->getMessage()]);
      return FALSE;
    }
    $this->state->delete(self::STATE_KEY_DUE_TIMESTAMP);
    return TRUE;
  }

  /**
   * Check if a trigger due date exists and is due, then call the trigger.
   */
  public function checkDueDate(): void {
    if (($dueTime = $this->state->get(self::STATE_KEY_DUE_TIMESTAMP)) && $dueTime <= $this->time->getRequestTime()) {
      $this->triggerExport();
    }
  }

}
